{*******************************************************}
{                                                       }
{       Turbo Pascal Version 4.0                        }
{       DOSCRT Unit                                     }
{                                                       }
{*******************************************************}



unit DOSCrt;

interface

Uses DOS;

const

  { CRT modes }
  BW40          = 0;            { 40x25 B/W on Color Adapter }
  CO40          = 1;            { 40x25 Color on Color Adapter }
  BW80          = 2;            { 80x25 B/W on Color Adapter }
  CO80          = 3;            { 80x25 Color on Color Adapter }
  Mono          = 7;            { 80x25 on Monochrome Adapter }
(*
  Font8x8       = 256;          { Add-in for ROM font }
*)
  { Mode constants for 3.0 compatibility }
  C40           = CO40;
  C80           = CO80;

  { Foreground and background color constants }
  Black         = 0;
  Blue          = 1;
  Green         = 2;
  Cyan          = 3;
  Red           = 4;
  Magenta       = 5;
  Brown         = 6;
  LightGray     = 7;

  { Foreground color constants }
  DarkGray      = 8;
  LightBlue     = 9;
  LightGreen    = 10;
  LightCyan     = 11;
  LightRed      = 12;
  LightMagenta  = 13;
  Yellow        = 14;
  White         = 15;

  { Add-in for blinking }
  Blink         = 128;

var
  CheckBreak: Boolean;    { Enable Ctrl-Break }
(*
  CheckEOF: Boolean;      { Enable Ctrl-Z }                        { NA }
  DirectVideo: Boolean;   { Enable direct video addressing }       { NA }
  CheckSnow: Boolean;     { Enable snow filtering }                { NA }
*)
  LastMode: Word;         { Current text mode }
  TextAttr: Byte;         { Current text attribute }
  WindMin: Word;          { Window upper left coordinates }
  WindMax: Word;          { Window lower right coordinates }
  SystemInt23 : pointer;  { Systems interrupt $23 }
  SaveInt1B: Pointer;     { Saved interrupt $1B }

function KeyPressed: Boolean;             { DOS  }
function ReadKey: Char;                   { DOS  }
procedure TextMode(Mode: Word);           { ANSI }
procedure GotoXY(X,Y: Byte);              { ANSI }
function WhereX: Byte;                    { ANSI }
function WhereY: Byte;                    { ANSI }
procedure ClrScr;                         { ANSI }
procedure ClrEol;                         { ANSI }
procedure TextColor(Color: Byte);         { ANSI }
procedure TextBackground(Color: Byte);    { ANSI }
procedure LowVideo;                       { ANSI }
procedure HighVideo;                      { ANSI }
procedure NormVideo;                      { ANSI } { Not Same Function }

implementation

Const
  ESCI = #27'[';

Var
  DOSCRT_OldExitProc : pointer;


{ Inline Macros }

{ This procedure will jump from an ISR to the ISR vector passed }
procedure JmpOldISR(OldISR: pointer);
  inline($5B/$58/$87/$5E/$0E/$87/$46/$10/$89/
         $EC/$5D/$07/$1F/$5F/$5E/$5A/$59/$CB);
{ This Function reads a character and echo's it back to the screen }
Function ReadDOSEcho:Char;
  inline($B4/$01/             { MOV AH,01    ; Code for Input    }
         $CD/$21/             { INT 21       ; Call DOS          }
         $50/                 { PUSH AX      ; Save Char         }
         $B4/$02/             { MOV AH,      ; Code for Output   }
         $88/$C2/             { MOV DL,AL    ; Move Char to DL   }
         $50/                 { INT 21       ; Call DOS          }
         $58);                { POP AX       ; Return Char in AX }

Function ReadDOSNoEcho:Char;
  inline($B4/$08/             { MOV AH,08    ; Code for Input    }
         $CD/$21);            { INT 21       ; Call DOS          }

Procedure WriteDOS( C : Char );
  inline($B4/$02/             { MOV AH,02    ; Code for Output   }
         $5A/                 { POP DX       ; Get Char to write }
         $CD/$21);            { INT 21       ; Call DOS          }

Function KeyReadyDos : Boolean;
  Inline($B4/$0B/             { MOV AH,0B    ; Code for Check    }
         $CD/$21/             { INT 21       ; Call DOS          }
         $25/$01/$00);        { AND AX,01    ; Mask First Bit    }


{ Private FAR Call Procedures and Functions }

{ Procedure DOSCrt_1B_23     Interrupt $1B and $23 Intermediate Handler }
{ Procedure DOSCrt_ExitProc  ExitCode for DOSCrt                        }

{$F+}

Procedure DOSCrt_1B_23;
Interrupt;
Begin
  Inline( $FA/$B0/$20/$E6/$20 );   { Reset Interrupt Control Port }
  If CheckBreak then
    Begin
       JmpOldISR( SystemInt23 );
    end;
end;

Procedure DOSCrt_ExitProc;
Begin
  ExitProc := DOSCrt_OldExitProc;
  SetIntVec( $23, SystemInt23 );
  SetIntVec( $1B, SaveInt1B );
end;

{$F-}


function KeyPressed: Boolean;
Begin
  KeyPressed := KeyReadyDOS;
end;

function ReadKey: Char;
Begin
  ReadKey := ReadDOSNoEcho;
end;

Procedure TextMode;
Begin
  Write( ESCI,'=',Mode,'h' );
  LastMode := Mode;
end;

procedure GotoXY(X,Y: Byte);
Begin
  Write( ESCI,X,';',Y,'H' );
end;

Function WhereX: Byte;
Var
  X,I: byte;
Begin
  Write( ESCI,'6n' );
  for I := 1 to 2 do
    if readkey = #0 then;
  X := (byte( readkey )- 48) * 10;
  WhereX := X + Byte( readkey ) - 48;
  for I := 1 to 5 do
    if readkey = #0 then;
end;

function WhereY: Byte;
Var
  Y,I: byte;
Begin
  Write( ESCI,'6n' );
  for I := 1 to 5 do
    if readkey = #0 then;
  Y := (byte( readkey ) - 48) * 10;
  WhereY := Y + Byte( readkey ) - 48;
  for I := 1 to 2 do
    if readkey = #0 then;
end;

Procedure ClrScr;
Begin
  Write( ESCI,'2J' );
End;

Procedure ClrEol;
begin
  Write( ESCI,'K' );
end;

Procedure SetTextAttr;
Begin
  Write( ESCI,'0;' );
  If ((TextAttr and $80) = $80) then
    Write( '5;' );
  if ((TextAttr and $08) = $08) then
    Write( '1;' );
  Case (TextAttr and $07) of
    0 : Write( '30;' );
    1 : Write( '34;' );
    2 : Write( '32;' );
    3 : Write( '36;' );
    4 : Write( '31;' );
    5 : Write( '35;' );
    6 : Write( '33;' );
    7 : Write( '37;' );
      Else
        Write( '0;' );
  end;
  Case (TextAttr and $70) shr 4 of
    0 : Write( '40' );
    1 : Write( '44' );
    2 : Write( '42' );
    3 : Write( '46' );
    4 : Write( '41' );
    5 : Write( '45' );
    6 : Write( '43' );
    7 : Write( '47' );
      Else
        Write( '0' );
  end;
  Write('m');
end;

procedure TextColor(Color: Byte);
Begin
  TextAttr := (TextAttr and $70) + (Color and $8F);
  SetTextAttr;
end;

procedure TextBackground(Color: Byte);
Begin
  TextAttr := (TextAttr and $8F) + ((Color shl 4) and $70);
  SetTextAttr;
end;

Procedure LowVideo;
Begin
  TextAttr := TextAttr and $F7;
  SetTextAttr;
End;

Procedure HighVideo;
Begin
  TextAttr := TextAttr or $8;
  SetTextAttr;
End;

Procedure NormVideo;
Begin
  TextAttr := 7;
  SetTextAttr;
end;

Begin { Initialization }
  GetIntVec( $23, SystemInt23 );
  GetIntVec( $1B, SaveInt1B );
  SetIntVec( $23, @DOSCrt_1B_23 );
  SetIntVec( $1B, @DOSCrt_1B_23 );
  DOSCrt_OldExitProc := ExitProc;
  ExitProc := @DOSCrt_ExitProc;
  CheckBreak := True;
  LastMode := CO80;
  TextAttr := 7;
  reset( input );
  rewrite( output );
end.
************************************************************************

{*******************************************************}
{                                                       }
{       Turbo Pascal Version 4.0                        }
{       DOSCRT Unit                                     }
{                                                       }
{       Copyright (c) 1988 Borland International, Inc.  }
{                                                       }
{*******************************************************}



unit DOSCrt;

interface

Uses DOS;

const

  { CRT modes }
  BW40          = 0;            { 40x25 B/W on Color Adapter }
  CO40          = 1;            { 40x25 Color on Color Adapter }
  BW80          = 2;            { 80x25 B/W on Color Adapter }
  CO80          = 3;            { 80x25 Color on Color Adapter }
  Mono          = 7;            { 80x25 on Monochrome Adapter }
(*
  Font8x8       = 256;          { Add-in for ROM font }
*)
  { Mode constants for 3.0 compatibility }
  C40           = CO40;
  C80           = CO80;

  { Foreground and background color constants }
  Black         = 0;
  Blue          = 1;
  Green         = 2;
  Cyan          = 3;
  Red           = 4;
  Magenta       = 5;
  Brown         = 6;
  LightGray     = 7;

  { Foreground color constants }
  DarkGray      = 8;
  LightBlue     = 9;
  LightGreen    = 10;
  LightCyan     = 11;
  LightRed      = 12;
  LightMagenta  = 13;
  Yellow        = 14;
  White         = 15;

  { Add-in for blinking }
  Blink         = 128;

var
  CheckBreak: Boolean;    { Enable Ctrl-Break }
(*
  CheckEOF: Boolean;      { Enable Ctrl-Z }                        { NA }
  DirectVideo: Boolean;   { Enable direct video addressing }       { NA }
  CheckSnow: Boolean;     { Enable snow filtering }                { NA }
*)
  LastMode: Word;         { Current text mode }
  TextAttr: Byte;         { Current text attribute }
  WindMin: Word;          { Window upper left coordinates }
  WindMax: Word;          { Window lower right coordinates }
  SystemInt23 : pointer;  { Systems interrupt $23 }
  SaveInt1B: Pointer;     { Saved interrupt $1B }

procedure AssignCrt(var F: Text);         { DOS  }
function KeyPressed: Boolean;             { DOS  }
function ReadKey: Char;                   { DOS  }
procedure TextMode(Mode: Word);           { ANSI }
(*
procedure Window(X1,Y1,X2,Y2: Byte);      { BIOS (Unavailable Through DOS) }
*)
procedure GotoXY(X,Y: Byte);              { ANSI }
function WhereX: Byte;                    { ANSI }
function WhereY: Byte;                    { ANSI }
procedure ClrScr;                         { ANSI }
procedure ClrEol;                         { ANSI }
(*
procedure InsLine;                        { BIOS (Unavailable Through DOS) }
procedure DelLine;                        { BIOS (Unavailable Through DOS) }
*)
procedure TextColor(Color: Byte);         { ANSI }
procedure TextBackground(Color: Byte);    { ANSI }
procedure LowVideo;                       { ANSI }
procedure HighVideo;                      { ANSI }
procedure NormVideo;                      { ANSI } { Not Same Function }
(*
procedure Delay(MS: Word);                { ???? (Unavailable Through DOS) }
procedure Sound(Hz: Word);                { ???? (Unavailable Through DOS) }
procedure NoSound;                        { ???? (Unavailable Through DOS) }
*)

implementation

Const
  ESCI = #27'[';

Var
  DOSCRT_OldExitProc : pointer;


{ Inline Macros }

{ This procedure will jump from an ISR to the ISR vector passed }
procedure JmpOldISR(OldISR: pointer);
  inline($5B/$58/$87/$5E/$0E/$87/$46/$10/$89/
         $EC/$5D/$07/$1F/$5F/$5E/$5A/$59/$CB);

Function ReadDOSEcho:Char;
  inline($B4/$01/             { MOV AH,01    ; Code for Input    }
         $CD/$21/             { INT 21       ; Call DOS          }
         $50/                 { PUSH AX      ; Save Char         }
         $B4/$02/             { MOV AH,      ; Code for Output   }
         $88/$C2/             { MOV DL,AL    ; Move Char to DL   }
         $50/                 { INT 21       ; Call DOS          }
         $58);                { POP AX       ; Return Char in AX }

Function ReadDOSNoEcho:Char;
  inline($B4/$08/             { MOV AH,08    ; Code for Input    }
         $CD/$21);            { INT 21       ; Call DOS          }

Procedure WriteDOS( C : Char );
  inline($B4/$02/             { MOV AH,02    ; Code for Output   }
         $5A/                 { POP DX       ; Get Char to write }
         $CD/$21);            { INT 21       ; Call DOS          }

Function KeyReadyDos : Boolean;
  Inline($B4/$0B/             { MOV AH,0B    ; Code for Check    }
         $CD/$21/             { INT 21       ; Call DOS          }
         $25/$01/$00);        { AND AX,01    ; Mask First Bit    }


{ Private FAR Call Procedures and Functions }

{ Procedure DOSCrt_1B_23     Interrupt $1B and $23 Intermediate Handler }
{ Procedure DOSCrt_ExitProc  ExitCode for DOSCrt                        }
{ Function  DOSCrt_Open      Text File Device Driver Open Function      }
{ Function  DOSCrt_In        Text File Device Driver Input Function     }
{ Function  DOSCrt_Out       Text File Device Driver Output Function    }
{ Function  DOSCrt_NUL       Text File Device Driver Close Function     }

{$F+}

Procedure DOSCrt_1B_23;
Interrupt;
Begin
  If CheckBreak then
    Begin
       inline($E4/$61/$8A/$E0/$0C/$80/$E6/$61/   { Clean up as the    }
              $86/$E0/$E6/$61/$B0/$20/$E6/$20);  { Bios would         }
       JmpOldISR( SystemInt23 );
    end;
end;

Procedure DOSCrt_ExitProc;
Begin
  ExitProc := DOSCrt_OldExitProc;
  SetIntVec( $23, SystemInt23 );
  SetIntVec( $1B, SaveInt1B );
end;

Function DOSCrt_NUL ( Var F: TextRec ): Integer;
Begin
  DOSCrt_NUL := 0;
end;

function DOSCrt_IN(var F: TextRec): Integer;
var
  P : Word;
  NotDone : Boolean;
begin
  with F do
  begin
    P:=0;
    NotDone := True;
    while NotDone and (P<BufSize) do
    begin
      BufPtr^[P] := ReadDOSEcho;
      if BufPtr^[P] = #13 then
        Begin
          Inc(P);
          BufPtr^[P] := #10;
          WriteDOS( #10 );
          NotDone := False;
        end;
      Inc(P);
    end;
    BufPos:=0; BufEnd:=P;
  end;
  DOSCrt_IN := 0;
end;

function DOSCrt_Out(var F: TextRec): Integer;
var
  P: Word;
begin
  with F do
  begin
    P:=0;
    while P<BufPos do
    begin
      WriteDOS( BufPtr^[P] );
      Inc(P);
    end;
    BufPos := 0;
  end;
    DOSCrt_Out := 0;
end;


Function DOSCrt_Open(var F: TextRec): Integer;
begin
  with F do
  begin
    if Mode=fmInput then
    begin
      InOutFunc:=@DOSCrt_In;
      FlushFunc:=@DOSCrt_NUL;
    end else
    begin
      Mode:=fmOutput;
      InOutFunc:=@DOSCrt_Out;
      FlushFunc:=@DOSCrt_Out;
    end;
    CloseFunc:=@DOSCrt_NUL;
  end;
  DOSCrt_Open:=0;
end;

{$F-}

procedure AssignCrt;
begin
  with TextRec(F) do
  begin
    Handle:=$FFFF;
    Mode:=fmClosed;
    BufSize:=SizeOf(Buffer);
    BufPtr:=@Buffer;
    BufEnd := 0;
    BufPos := 0;
    OpenFunc:=@DOSCrt_Open;
    Name[0]:=#0;
  end;
end;


function KeyPressed: Boolean;
Begin
  KeyPressed := KeyReadyDOS;
end;

function ReadKey: Char;
Begin
  ReadKey := ReadDOSNoEcho;
end;

Procedure TextMode;
Begin
  Write( ESCI,'=',Mode,'h' );
  LastMode := Mode;
end;

procedure GotoXY(X,Y: Byte);
Begin
  Write( ESCI,X,';',Y,'H' );
end;

Function WhereX: Byte;
Var
  X,I: byte;
Begin
  Write( ESCI,'6n' );
  for I := 1 to 2 do
    if readkey = #0 then;
  X := (byte( readkey )- 48) * 10;
  WhereX := X + Byte( readkey ) - 48;
  for I := 1 to 5 do
    if readkey = #0 then;
end;

function WhereY: Byte;
Var
  Y,I: byte;
Begin
  Write( ESCI,'6n' );
  for I := 1 to 5 do
    if readkey = #0 then;
  Y := (byte( readkey ) - 48) * 10;
  WhereY := Y + Byte( readkey ) - 48;
  for I := 1 to 2 do
    if readkey = #0 then;
end;

Procedure ClrScr;
Begin
  Write( ESCI,'2J' );
End;

Procedure ClrEol;
begin
  Write( ESCI,'K' );
end;

Procedure SetTextAttr;
Begin
  Write( ESCI,'0;' );
  If ((TextAttr and $80) = $80) then
    Write( '5;' );
  if ((TextAttr and $08) = $08) then
    Write( '1;' );
  Case (TextAttr and $07) of
    0 : Write( '30;' );
    1 : Write( '34;' );
    2 : Write( '32;' );
    3 : Write( '36;' );
    4 : Write( '31;' );
    5 : Write( '35;' );
    6 : Write( '33;' );
    7 : Write( '37;' );
      Else
        Write( '0;' );
  end;
  Case (TextAttr and $70) shr 4 of
    0 : Write( '40' );
    1 : Write( '44' );
    2 : Write( '42' );
    3 : Write( '46' );
    4 : Write( '41' );
    5 : Write( '45' );
    6 : Write( '43' );
    7 : Write( '47' );
      Else
        Write( '0' );
  end;
  Write('m');
end;

procedure TextColor(Color: Byte);
Begin
  TextAttr := (TextAttr and $70) + (Color and $8F);
  SetTextAttr;
end;

procedure TextBackground(Color: Byte);
Begin
  TextAttr := (TextAttr and $8F) + ((Color shl 4) and $70);
  SetTextAttr;
end;

Procedure LowVideo;
Begin
  TextAttr := TextAttr and $F7;
  SetTextAttr;
End;

Procedure HighVideo;
Begin
  TextAttr := TextAttr or $8;
  SetTextAttr;
End;

Procedure NormVideo;
Begin
  TextAttr := 7;
  SetTextAttr;
end;

Begin { Initialization }
  GetIntVec( $23, SystemInt23 );
  GetIntVec( $1B, SaveInt1B );
  SetIntVec( $23, @DOSCrt_1B_23 );
  SetIntVec( $1B, @DOSCrt_1B_23 );
  DOSCrt_OldExitProc := ExitProc;
  ExitProc := @DOSCrt_ExitProc;
  CheckBreak := True;
  AssignCrt( input );
  AssignCrt( output );
  reset( input );
  rewrite( output );
  LastMode := CO80;
  TextAttr := 7;
end.
